/*----------------------------------------------------------------------------*/
/* Copyright (c) FIRST 2008. All Rights Reserved.                             */
/* Open Source Software - may be modified and shared by FRC teams. The code   */
/* must be accompanied by the FIRST BSD license file in the root directory of */
/* the project.                                                               */
/*----------------------------------------------------------------------------*/

package edu.wpi.first.wpilibj;

/**
 * A robot base class. Users can build thier program off of this by overring the
 * autonomous and operatorControl methods.
 * @todo If this is going to last until release, it needs a better name.
 */
public class SimpleRobot extends RobotBase {

    private boolean m_robotMainOverridden;

    /**
     * Create a new SimpleRobot
     */
    public SimpleRobot() {
        super();
        m_robotMainOverridden = true;
    }

    /**
     * Autonomous should go here.
     * Users should add autonomous code to this method that should run while the field is
     * in the autonomous period.
     */
    public void autonomous() {
        System.out.println("Provided Autonomous() method running");
    }

    /**
     * Operator control (tele-operated) code should go here.
     * Users should add Operator Control code to this method that should run while the field is
     * in the Operator Control (tele-operated) period.
     */
    public void operatorControl() {
        System.out.println("Provided OperatorControl() method running");
    }

    /**
     * Robot main program for free-form programs.
     *
     * This should be overridden by user subclasses if the intent is to not use the autonomous() and
     * operatorControl() methods. In that case, the program is responsible for sensing when to run
     * the autonomous and operator control functions in their program.
     *
     * This method will be called immediately after the constructor is called. If it has not been
     * overridden by a user subclass (i.e. the default version runs), then the autonomous() and
     * operatorControl() methods will be called.
     */
    public void robotMain() {
        System.out.println("Information: No user-supplied robotMain()");
        m_robotMainOverridden = false;
    }

    /**
     * Start a competition.
     * This code needs to track the order of the field starting to ensure that everything happens
     * in the right order. Currently it waits for the user autonomous code to complete before running
     * the OperatorControl function. Needs to spawn the autonomous code as a task, then kill it after
     * the end of the autonomous period.
     */
    public void startCompetition() {
        robotMain();
        if (!m_robotMainOverridden) {
            while (true) {
                // Wait for robot to be enabled
                while (isDisabled()) {
                    Timer.delay(.01);
                }
                // Now enabled - check if we should run Autonomous code
                if (isAutonomous()) {
                    autonomous();
                    while (isAutonomous() && !isDisabled()) {
                        Timer.delay(.01);
                    }
                } else {
                    operatorControl();				// run the operator control method
                    while (isOperatorControl() && !isDisabled()) {
                        Timer.delay(.01);
                    }
                }
            }
        }
    }
}
