/*----------------------------------------------------------------------------*/
/* Copyright (c) FIRST 2008. All Rights Reserved.                             */
/* Open Source Software - may be modified and shared by FRC teams. The code   */
/* must be accompanied by the FIRST BSD license file in the root directory of */
/* the project.                                                               */
/*----------------------------------------------------------------------------*/

package edu.wpi.first.wpilibj;

import edu.wpi.first.wpilibj.fpga.tGlobal;

/**
 * Contains global utility functions
 */
public class Utility {

    private Utility() {
    }

    /**
     * Return the FPGA Version number.
     * For now, expect this to be 2009.
     * @return FPGA Version number.
     */
    int getFPGAVersion() {
        return tGlobal.readVersion();
    }

    /**
     * Return the FPGA Revision number.
     * The format of the revision is 3 numbers.
     * The 12 most significant bits are the Major Revision.
     * the next 8 bits are the Minor Revision.
     * The 12 least significant bits are the Build Number.
     * @return FPGA Revision number.
     */
    long getFPGARevision() {
        return tGlobal.readRevision();
    }

    /**
     * Read the microsecond timer from the FPGA.
     *
     * @return The current time in microseconds according to the FPGA.
     */
    public static long getFPGATime() {
        return tGlobal.readLocalTime();
    }
}
